/****************************************************************************
* VCGLib                                                            o o     *
* Visual and Computer Graphics Library                            o     o   *
*                                                                _   O  _   *
* Copyright(C) 2004-2009                                           \/)\/    *
* Visual Computing Lab                                            /\/|      *
* ISTI - Italian National Research Council                           |      *
*                                                                    \      *
* All rights reserved.                                                      *
*                                                                           *
* This program is free software; you can redistribute it and/or modify      *
* it under the terms of the GNU General Public License as published by      *
* the Free Software Foundation; either version 2 of the License, or         *
* (at your option) any later version.                                       *
*                                                                           *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
* GNU General Public License (http://www.gnu.org/licenses/gpl.txt)          *
* for more details.                                                         *
*                                                                           *
****************************************************************************/

#include<vcg/simplex/vertex/base.h>
#include<vcg/simplex/vertex/component.h>

#include <vcg/complex/used_types.h>

#include<vcg/simplex/face/base.h>
#include<vcg/simplex/face/component.h>
#include<vcg/complex/complex.h>
#include<vcg/complex/algorithms/create/platonic.h>
#include<wrap/io_trimesh/import_ply.h>

using namespace vcg;
using namespace std;

class MyEdge;
class MyFace;
class MyVertex;
struct MyUsedTypes : public UsedTypes<	Use<MyVertex>   ::AsVertexType,
                                        Use<MyEdge>     ::AsEdgeType,
                                        Use<MyFace>     ::AsFaceType>{};

class MyVertex  : public Vertex<MyUsedTypes, vertex::Coord3f, vertex::Normal3f, vertex::BitFlags  >{};
class MyFace    : public Face< MyUsedTypes, face::VertexRef, face::BitFlags, face::FFAdj > {};
class MyEdge    : public Edge<MyUsedTypes>{};
class MyMesh    : public tri::TriMesh< vector<MyVertex>, vector<MyFace> , vector<MyEdge>  > {};


int main( int argc, char **argv )
{
  MyMesh m;
  if(argc < 2 ) return -1;

  printf("Reading %s\n",argv[1]);
  int ret= tri::io::ImporterPLY<MyMesh>::Open(m,argv[1]);
  if(ret!=0)
  {
    printf("Unable to open %s for '%s'\n",argv[1],tri::io::ImporterPLY<MyMesh>::ErrorMsg(ret));
    return -1;
  }

  tri::UpdateTopology<MyMesh>::FaceFace(m);
  printf("Mesh has %i vert and %i faces\n",m.vn, m.fn);
  tri::UpdateFlags<MyMesh>::VertexClearV(m);

  for(MyMesh::FaceIterator fi=m.face.begin();fi!=m.face.end();++fi)
  {
    for(int i=0;i<3;++i)
      if(fi->FFp(i)==&*fi && !fi->V(i)->IsV())
      {
        int borderSize=0;
        face::Pos<MyFace> pp(&*fi,i,fi->V(i));
        assert(pp.IsBorder());
        MyMesh::VertexPointer startVertex = pp.V();
        do
        {
          pp.V()->SetV();
          pp.FlipE();
          while(!pp.IsBorder())
          {
            pp.FlipF();
            pp.FlipE();
          }
          pp.FlipV();
          borderSize++;
        } while(pp.V() != startVertex);

        printf("found one loop of %i edges\n",borderSize);
      }
  }
//  printf("Found %i border edges\n",border);
  return 0;
}
